@@ -3,6 +3,8 @@ import subprocess
 import shlex
 from typing import Union
 
+from .exceptions import *
+
 PathLike = Union[str, Path]
 
 
@@ -18,21 +20,5 @@ def call_git(command: str, cwd=None, wrap=False) -> str:
         raise
 
 
-class HashNotFound(Exception):
-    pass
-
-
-class RepositoryError(Exception):
-    pass
-
-
-class RepositoryNotFound(RepositoryError):
-    pass
-
-
-class InconsistentRepositories(RepositoryError):
-    pass
-
-
 # legacy
 HashNotFoundError = HashNotFound
